SystemOrganization addCategory: #'GT-Spotter-EventRecorder-Server'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Server-Core'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Server-Tests'!

Object subclass: #GTSpotterEventInfo
	instanceVariableNames: 'directory data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Core'!

!GTSpotterEventInfo methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 17:24'!
data
	^ data ifNil: [ data := GTSpotterEventRecorder materializeObjectsFromDirectory: self directory ]! !

!GTSpotterEventInfo methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 17:24'!
directory
	^ directory! !

!GTSpotterEventInfo methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 17:22'!
directory: aFileReference 
	directory := aFileReference! !

!GTSpotterEventInfo methodsFor: 'requesting' stamp: 'JurajKubelka 3/16/2015 17:36'!
get
	^ String streamContents: [ :aStream |
		aStream
			nextPutAll: 'Number of packages: ';
			print: self data size;
			cr;
			nextPutAll: 'Number of computers: ';
			print: self numberOfComputers;
			cr
		 ]! !

!GTSpotterEventInfo methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 17:34'!
numberOfComputers
	^ self perComputer size! !

!GTSpotterEventInfo methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 17:34'!
perComputer
	^ self data groupedBy: #computerUUID! !

Object subclass: #GTSpotterRecorderObjectToFileStorage
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Core'!

!GTSpotterRecorderObjectToFileStorage class methodsFor: 'instance creation' stamp: 'JurajKubelka 2/6/2015 12:17'!
on: directory
	^ self new
		directory: directory;
		yourself! !

!GTSpotterRecorderObjectToFileStorage methodsFor: 'accessing' stamp: 'JurajKubelka 2/6/2015 12:37'!
at: key
	^ self at: key ifAbsent: [ KeyNotFound signalFor: key ]! !

!GTSpotterRecorderObjectToFileStorage methodsFor: 'accessing' stamp: 'JurajKubelka 2/7/2015 12:15'!
at: key ifAbsent: block
	| fullPath |
	fullPath := self directory / (key first: 2) / key.
	fullPath exists
		ifFalse: [ ^ block value ].
	fullPath size isZero 
		ifTrue: [ ^ block value ].
	^ fullPath binaryReadStreamDo: [ :stream | 
			stream upToEnd ]! !

!GTSpotterRecorderObjectToFileStorage methodsFor: 'accessing' stamp: 'JurajKubelka 2/6/2015 15:23'!
at: key put: aByteArray
	| subDirectory file |
	self directory ensureCreateDirectory.
	subDirectory := self directory / (key first: 2).
	subDirectory ensureCreateDirectory.
	file := (subDirectory / key).
	file ensureDelete.
	file writeStreamDo: [ :stream | 
		stream nextPutAll: aByteArray ]
! !

!GTSpotterRecorderObjectToFileStorage methodsFor: 'accessing' stamp: 'JurajKubelka 2/6/2015 12:18'!
directory
	^ directory! !

!GTSpotterRecorderObjectToFileStorage methodsFor: 'accessing' stamp: 'JurajKubelka 2/6/2015 12:18'!
directory: aDirectory
	directory := aDirectory! !

Object subclass: #GTSpotterRecorderServer
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Core'!

!GTSpotterRecorderServer class methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 16:42'!
start
	"self start"
	(ZnServer startDefaultOn: 8080)
		delegate: GTSpotterRecorderServerDelegate new! !

!GTSpotterRecorderServer class methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 16:44'!
stop
	"self stop"
	ZnServer stopDefault! !

Object subclass: #GTSpotterRecorderServerDelegate
	instanceVariableNames: 'compressedEventFile prefix storage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Core'!

!GTSpotterRecorderServerDelegate class methodsFor: 'instance creation' stamp: 'JurajKubelka 2/26/2015 16:07'!
on: storage
	^ self new
		storage: storage;
		yourself! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 3/17/2015 10:57'!
compressedEventFile
	^ compressedEventFile! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 3/17/2015 10:57'!
compressedEventFile: anObject
	compressedEventFile := anObject! !

!GTSpotterRecorderServerDelegate methodsFor: 'private' stamp: 'JurajKubelka 2/26/2015 15:46'!
generateNewKey
	^ UUID new asString! !

!GTSpotterRecorderServerDelegate methodsFor: 'request handling' stamp: 'JurajKubelka 3/17/2015 10:57'!
handleGetDataRequest: request 
	^ self handleGetDataRequest: request fromFile: compressedEventFile! !

!GTSpotterRecorderServerDelegate methodsFor: 'request handling' stamp: 'JurajKubelka 3/17/2015 10:56'!
handleGetDataRequest: request fromFile: filename
	| file entry size time mimeType fileStream entity response modified |
	file := filename asFileReference.
	entry := file entry.
	size := entry size.
	time := entry modificationTime.
	modified := true.
	request headers
		at: 'If-Modified-Since'
		ifPresent: [ modified := time > (ZnUtils parseHttpDate: (request headers at: 'If-Modified-Since')) ].
	modified
		ifTrue: [ 
			mimeType := ZnMimeType forFilenameExtension: file extension.
			fileStream := file readStream.
			fileStream binary.
			(entity := ZnStreamingEntity type: mimeType length: size) 
				stream: fileStream.
			(response := ZnResponse ok: entity) 
				headers at: 'Modification-Date' put: (ZnUtils httpDate: time) ]
		ifFalse: [ response := ZnResponse notModified ].
	^ response! !

!GTSpotterRecorderServerDelegate methodsFor: 'request handling' stamp: 'JurajKubelka 3/16/2015 17:38'!
handleGetInfoRequest: request 
	^ ZnResponse ok: (ZnEntity text: (
		GTSpotterEventInfo new 
			directory: self storage directory;
			get))! !

!GTSpotterRecorderServerDelegate methodsFor: 'request handling' stamp: 'JurajKubelka 3/16/2015 17:40'!
handleGetRequest: request

	(request uri segments endsWith: #('gt' 'events' 'info')) 
		ifTrue: [ ^ self handleGetInfoRequest: request ].

	(request uri segments endsWith: #('gt' 'events' 'data')) 
		ifTrue: [ ^ self handleGetDataRequest: request ].

	^ ZnResponse ok: (ZnEntity text: 'OK')
	! !

!GTSpotterRecorderServerDelegate methodsFor: 'request handling' stamp: 'JurajKubelka 2/26/2015 16:24'!
handlePutRequest: request
	| badRequest key |
	badRequest := [ ^ ZnResponse badRequest: request ].
	(request hasEntity and: [ request contentType matches: ZnMimeType applicationOctetStream ])
		ifFalse: badRequest.
	[ key := self saveByteArray: request entity bytes ] on: Error do: badRequest.
	^ ZnResponse created: key! !

!GTSpotterRecorderServerDelegate methodsFor: 'public' stamp: 'JurajKubelka 2/26/2015 15:55'!
handleRequest: request

	request method = #PUT
		ifTrue: [ 
			^ self handlePutRequest: request ].
	request method = #GET		
		ifTrue: [ 
			^ self handleGetRequest: request ].
	^ ZnResponse notFound: request uri! !

!GTSpotterRecorderServerDelegate methodsFor: 'initialization' stamp: 'JurajKubelka 3/17/2015 11:41'!
initialize
	super initialize.
	storage := GTSpotterRecorderObjectToFileStorage on: FileLocator home / 'gt' / 'events'.
	compressedEventFile := FileLocator home / 'public_html' / 'gt-events.tgz'.
	prefix := #().! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 15:31'!
prefix
	^ prefix! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 15:32'!
prefix: aCollectionOfStrings
	prefix := aCollectionOfStrings! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 15:32'!
prefixFromString: string
	"Set the path prefix under which I should be handling request to string,
	interpreting each /-separated token as a path element"

	self prefix: (string findTokens: '/')! !

!GTSpotterRecorderServerDelegate methodsFor: 'private' stamp: 'JurajKubelka 2/26/2015 15:53'!
saveByteArray: aByteArray
	| key |
	key := self generateNewKey.
	self storage at: key put: aByteArray.
	^ key! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 16:06'!
storage
	^ storage! !

!GTSpotterRecorderServerDelegate methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 16:06'!
storage: aStorage
	storage := aStorage! !

TestCase subclass: #GTSpotterRecorderObjectToFileStorageTest
	instanceVariableNames: 'storage fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Tests'!

!GTSpotterRecorderObjectToFileStorageTest commentStamp: '<historical>' prior: 0!
A RAObjectToFileStorageTest is a test class for testing the behavior of RAObjectToFileStorage!

!GTSpotterRecorderObjectToFileStorageTest methodsFor: 'running' stamp: 'JurajKubelka 2/26/2015 15:24'!
setUp
	super setUp.
	fileSystem := FileSystem memory.
	storage := GTSpotterRecorderObjectToFileStorage on: fileSystem workingDirectory.! !

!GTSpotterRecorderObjectToFileStorageTest methodsFor: 'running' stamp: 'JurajKubelka 2/6/2015 15:00'!
tearDown
	fileSystem close.
	super tearDown.
! !

!GTSpotterRecorderObjectToFileStorageTest methodsFor: 'tests' stamp: 'JurajKubelka 2/6/2015 15:21'!
testAtPut
	| uuid input output |
	uuid := UUID new asString.
	input := #[ 1 2 3 4 ].
	storage at: uuid put: input.
	output := storage at: uuid.
	self assert: output equals: input.
! !

!GTSpotterRecorderObjectToFileStorageTest methodsFor: 'tests' stamp: 'JurajKubelka 2/6/2015 15:22'!
testAtPut2
	| uuid input output |
	uuid := UUID new asString.
	input := #[ 1 2 3 4 ].
	storage at: uuid put: input.
	output := storage at: uuid.
	self assert: output equals: input.

	input := #[ 5 6 ].
	storage at: uuid put: input.
	output := storage at: uuid.
	self assert: output equals: input.! !

!GTSpotterRecorderObjectToFileStorageTest methodsFor: 'tests' stamp: 'JurajKubelka 2/6/2015 15:02'!
testDirectory
	self assert: storage directory exists.
	self assert: storage directory isWritable.! !

TestCase subclass: #GTSpotterRecorderServerDelegateTest
	instanceVariableNames: 'fileSystem storage delegate server client data location'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Server-Tests'!

!GTSpotterRecorderServerDelegateTest commentStamp: '<historical>' prior: 0!
A GTSpotterRecorderServerDelegateTest is a test class for testing the behavior of GTSpotterRecorderServerDelegate!

!GTSpotterRecorderServerDelegateTest methodsFor: 'running' stamp: 'JurajKubelka 2/26/2015 16:18'!
setUp
	super setUp.
	fileSystem := FileSystem memory.
	storage := GTSpotterRecorderObjectToFileStorage on: fileSystem workingDirectory.
	delegate := GTSpotterRecorderServerDelegate on: storage.
	server := ZnServer startOn: 1818 + 10 atRandom.
	server delegate: delegate.
	server debugMode: true; logToTranscript. 
	client := ZnClient new
		url: server localUrl;
		followRedirects: false;
		numberOfRetries: 0;
		enforceHttpSuccess: true;
		logToTranscript;
		yourself.! !

!GTSpotterRecorderServerDelegateTest methodsFor: 'running' stamp: 'JurajKubelka 2/26/2015 16:13'!
tearDown
	client close.
	server stop.
	fileSystem close.
	super tearDown.
! !

!GTSpotterRecorderServerDelegateTest methodsFor: 'tests' stamp: 'JurajKubelka 3/17/2015 11:17'!
testHandleGetData
	| contents file  |
	file := fileSystem workingDirectory / 'file.bin'.
	data := #[ 1 2 3 4 ].
	file ensureDelete.
	file writeStreamDo: [ :aStream |
		data do: [ :each | aStream nextPut: each ] ].
	delegate compressedEventFile: file.
	client
		path: '/gt/events/data';
		get.
	self assert: client isSuccess.
	contents := client response contents.
	self assert: contents equals: data.! !

!GTSpotterRecorderServerDelegateTest methodsFor: 'tests' stamp: 'JurajKubelka 3/16/2015 17:00'!
testHandleGetInfo
	| contents |
	client
		path: '/gt/events/info';
		get.
	self assert: client isSuccess.
	contents := client response contents.
	self assert: contents isString.
	self deny: contents = 'OK'.! !

!GTSpotterRecorderServerDelegateTest methodsFor: 'tests' stamp: 'JurajKubelka 2/26/2015 16:25'!
testHandlePut
	data := #[1 2 3 4].
	client
		entity: (ZnByteArrayEntity bytes: data);
		put.
	self assert: client isSuccess.
	self assert: client isCreated.
	location := client response location asZnUrl.
	self assert: (storage at: location lastPathSegment) equals: data.
! !
